@@ -17,14 +17,24 @@ not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 Cambridge, MA 02139, USA.  */
 
 #ifndef	_FNMATCH_H
-
 #define	_FNMATCH_H	1
 
+/* We #undef these before defining them because some losing systems
+   (HP-UX A.08.07 for example) define these in <unistd.h>.  */
+#undef  FNM_PATHNAME
+#undef  FNM_NOESCAPE
+#undef  FNM_PERIOD
+
 /* Bits set in the FLAGS argument to `fnmatch'.  */
-#define	FNM_PATHNAME	(1 << 0)/* No wildcard can ever match `/'.  */
-#define	FNM_NOESCAPE	(1 << 1)/* Backslashes don't quote special chars.  */
-#define	FNM_PERIOD	(1 << 2)/* Leading `.' is matched only explicitly.  */
-#define	__FNM_FLAGS	(FNM_PATHNAME|FNM_NOESCAPE|FNM_PERIOD)
+/* standard flags */
+#define	FNM_PATHNAME	(1 << 0) /* No wildcard can ever match `/'.  */
+#define	FNM_NOESCAPE	(1 << 1) /* Backslashes don't quote special chars.  */
+#define	FNM_PERIOD	(1 << 2) /* Leading `.' is matched only explicitly.  */
+
+/* extended flags */
+#define FNM_LEADING_DIR	(1 << 3) /* Ignore `/...' after a match. */
+#define FNM_CASEFOLD	(1 << 4) /* Compare without regard to case. */
+#define FNM_EXTMATCH	(1 << 5) /* Use ksh-like extended matching. */
 
 /* Value returned by `fnmatch' if STRING does not match PATTERN.  */
 #define	FNM_NOMATCH	1
@@ -33,4 +43,4 @@ Cambridge, MA 02139, USA.  */
    returning zero if it matches, FNM_NOMATCH if not.  */
 extern int fnmatch();
 
-#endif /* fnmatch.h */
+#endif /* _FNMATCH_H */
